"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function getScheme(config) {
    return typeof config.scheme === 'string' ? config.scheme : null;
}
exports.getScheme = getScheme;
function setScheme(config, infoPlist) {
    let scheme = getScheme(config);
    if (!scheme) {
        return infoPlist;
    }
    return Object.assign(Object.assign({}, infoPlist), { CFBundleURLTypes: [{ CFBundleURLSchemes: [scheme] }] });
}
exports.setScheme = setScheme;
// TODO: update this to work well idempotently!
function appendScheme(scheme, infoPlist) {
    if (!scheme) {
        return infoPlist;
    }
    let existingSchemes = infoPlist.CFBundleURLTypes;
    // No need to append if we don't have any
    if (!existingSchemes) {
        return setScheme({ scheme }, infoPlist);
    }
    return Object.assign(Object.assign({}, infoPlist), { CFBundleURLTypes: [
            ...existingSchemes,
            {
                CFBundleURLSchemes: [scheme],
            },
        ] });
}
exports.appendScheme = appendScheme;
function removeScheme(scheme, infoPlist) {
    if (!scheme) {
        return infoPlist;
    }
    // No need to remove if we don't have any
    if (!infoPlist.CFBundleURLTypes) {
        return infoPlist;
    }
    infoPlist.CFBundleURLTypes = infoPlist.CFBundleURLTypes.map(bundleUrlType => {
        const index = bundleUrlType.CFBundleURLSchemes.indexOf(scheme);
        if (index > -1) {
            bundleUrlType.CFBundleURLSchemes.splice(index, 1);
            if (bundleUrlType.CFBundleURLSchemes.length === 0) {
                return undefined;
            }
        }
        return bundleUrlType;
    }).filter(Boolean);
    return infoPlist;
}
exports.removeScheme = removeScheme;
function hasScheme(scheme, infoPlist) {
    const existingSchemes = infoPlist.CFBundleURLTypes;
    if (!Array.isArray(existingSchemes))
        return false;
    return existingSchemes.some(({ CFBundleURLSchemes: schemes }) => schemes.includes(scheme));
}
exports.hasScheme = hasScheme;
function getSchemesFromPlist(infoPlist) {
    if (Array.isArray(infoPlist.CFBundleURLTypes)) {
        return infoPlist.CFBundleURLTypes.reduce((schemes, { CFBundleURLSchemes }) => {
            if (Array.isArray(CFBundleURLSchemes)) {
                return [...schemes, ...CFBundleURLSchemes];
            }
            return schemes;
        }, []);
    }
    return [];
}
exports.getSchemesFromPlist = getSchemesFromPlist;
//# sourceMappingURL=Scheme.js.map