"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isStandaloneContextDataUser = isStandaloneContextDataUser;
exports.isStandaloneContextTestEnvironment = isStandaloneContextTestEnvironment;
exports.isStandaloneContextDataService = isStandaloneContextDataService;
exports.default = exports.StandaloneContextService = exports.StandaloneContextUser = void 0;

function _StandaloneBuildFlags() {
  const data = _interopRequireDefault(require("./StandaloneBuildFlags"));

  _StandaloneBuildFlags = function () {
    return data;
  };

  return data;
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

// currently unused
function isStandaloneContextDataUser(value) {
  return value && typeof value.projectPath === 'string' && 'exp' in value;
}

function isStandaloneContextTestEnvironment(value) {
  return ['none', 'local', 'ci'].includes(value);
}

function isStandaloneContextDataService(value) {
  return value && isStandaloneContextTestEnvironment(value.testEnvironment) && typeof value.expoSourcePath === 'string' && typeof value.shellAppSdkVersion === 'string';
}
/**
 *  A user context is used when we are configuring a standalone app locally on a user's machine,
 *  such as during `exp detach`.
 */


class StandaloneContext {
  constructor() {
    _defineProperty(this, "data", void 0);

    _defineProperty(this, "config", void 0);

    _defineProperty(this, "isAnonymous", () => {
      return this instanceof StandaloneContextService && !this.config;
    });
  }

}

_defineProperty(StandaloneContext, "createUserContext", (projectPath, exp, publishedUrl) => {
  const context = new StandaloneContextUser({
    projectPath,
    exp
  }, {
    url: publishedUrl,
    releaseChannel: 'default'
  }, // we never expect to handle the build step for user contexts right now
  _StandaloneBuildFlags().default.createEmpty());
  context.config = exp;
  return context;
});

_defineProperty(StandaloneContext, "createServiceContext", (expoSourcePath, archivePath, manifest, privateConfig, testEnvironment, build, publishedUrl, releaseChannel, shellAppSdkVersion) => {
  const context = new StandaloneContextService({
    expoSourcePath,
    archivePath,
    manifest,
    privateConfig,
    testEnvironment,
    shellAppSdkVersion
  }, {
    url: publishedUrl,
    releaseChannel: releaseChannel ? releaseChannel : 'default'
  }, build);
  context.config = manifest;
  return context;
});

class StandaloneContextUser extends StandaloneContext {
  constructor(data, published, build) {
    super();
    this.data = data;
    this.published = published;
    this.build = build;

    _defineProperty(this, "type", 'user');
  }

}

exports.StandaloneContextUser = StandaloneContextUser;

class StandaloneContextService extends StandaloneContext {
  constructor(data, published, build) {
    super();
    this.data = data;
    this.published = published;
    this.build = build;

    _defineProperty(this, "type", 'service');
  }

}

exports.StandaloneContextService = StandaloneContextService;
var _default = StandaloneContext;
exports.default = _default;
//# sourceMappingURL=../__sourcemaps__/detach/StandaloneContext.js.map
